'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _loglevel = require('loglevel');

var _loglevel2 = _interopRequireDefault(_loglevel);

var _bluebird = require('bluebird');

var _underscore = require('underscore.lifted');

var _underscore2 = _interopRequireDefault(_underscore);

var _jquery = require('jquery');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaultErrorHandler = function defaultErrorHandler(e) {
  _loglevel2.default.error('Error received in transactor');
  _loglevel2.default.error(e);
  _loglevel2.default.error(e.stack);
};

/**
 * Simple convention for executing an ajax request in a
 * reducer and creating a promise for the result.
 */

exports.default = function (route, resultHandler) {
  var errorHandler = arguments.length <= 2 || arguments[2] === undefined ? defaultErrorHandler : arguments[2];
  return function (dispatch, getState) {
    return _underscore2.default.lift((0, _underscore2.default)(route).isFunction()).map(function () {
      return route(getState());
    }).orElse(function () {
      return route;
    }).map(function (fetchRoute) {
      return new _bluebird.Promise(function (resolve, reject) {
        return (0, _jquery.ajax)(fetchRoute).then(function (resp) {
          return _underscore2.default.lift(resp.errors).map(function (errs) {
            return reject(errs);
          }).orElse(function () {
            return resolve(resp);
          }).unlift();
        });
      })
      // feed the server response into the appropriate handler, and dispatch.
      .then(_underscore2.default.compose(dispatch, resultHandler)).catch(errorHandler);
    }).unlift();
  };
};